@@ -1,17 +1,10 @@
-apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE, plot=TRUE, files=TRUE, Dyad=FALSE, dir) {
+apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE, plot=TRUE, Dyad=FALSE) {
   
   ################################################################################################
   # Parse the tab-delimited input file and convert genotypic states to numeric genotypic classes, 
   # based on primary and secondary alleles across the population.
   ################################################################################################
 
-  # Some previous definitions
-  pos = 1
-  envir = as.environment(pos)
-  options(warn=-1)
-  ptm <- proc.time()
-  
-  print("Importing data...")
   GK <- cbind(as.data.frame(InputFile[,1]),as.data.frame(InputFile[,2]))
   colnames(GK) <- c("genos","key")
   Data <- t(as.data.frame(InputFile[,3:ncol(InputFile)]))
@@ -103,8 +96,6 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
   # (GD) between each EPij and each potential Offspring (j) in the population (Offj).
   ###############################################################################################
   
-  print("Creating the EP(ij)'s and estimating the GDij|POk...")
-  
   # Load intermediate files, construct output vectors and initialize progress bar
   Parent1List <- vector(mode="numeric",length=0)
   Parent2List <- vector(mode="numeric",length=0)
@@ -188,20 +179,12 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
   # Output 1 - All triads
   Out1 <- data.frame(Parent1List,Parent2List,ObsProgList,TypeList,SNPsNumber,GD)
   colnames(Out1) <- c("Parent1","Parent2","Offspring","Cross.Type","SNPs","GD")
-  Triad_All <- Out1
-  assign("Triad_All", Out1, envir = envir)
-  if (files==TRUE){
-    if(!missing(dir)){
-      write.csv(Triad_All, file.path(dir, "apparent-Triad-All.csv"),row.names=F)
-    }
-  }
 
   #########################################################################
   # Parsing and reporting the significatives of the Triad analysis.
   # Finding the triad GAP and testing its significance and calculates the 
   # p-values for the significant triads.
   #########################################################################
-  print("Finding the Triad GAP, testing its significance and assign p-values for the significant Triad hits, if found....")
 
   # The log report files
   OMeanGD <- mean(Out1$GD)
@@ -210,15 +193,14 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
   OSdSNPs <- sd(Out1$SNPs)
   LogOut1 <- data.frame(OMeanGD,OSdGD,OMeanSNPs,OSdSNPs)
   colnames(LogOut1) <- c("Overall mean GDij|POk","Standard deviation GDij|POk","Overall mean usable loci","Standard deviation usable loci")
-  assign("Triad_Log1", LogOut1, envir = envir)
-  
+
   Geno <- vector(mode="numeric",length=0)
   MeanGDGeno <- vector(mode="numeric",length=0)
   MinGDGeno <- vector(mode="numeric",length=0)
   MaxGDGeno <- vector(mode="numeric",length=0)
   MeanSNPsGeno <- vector(mode="numeric",length=0)
   for (i in 1:nrow(GK)) {
-    genoOut <- subset (Out1, Out1$Parent1 == GK$genos[i])
+    genoOut <- Out1[which(Out1$Parent1 == GK$genos[i]),]
     Geno <- append(Geno,as.character(GK$genos[i]))
     MeanGDGeno <- append(MeanGDGeno,mean(genoOut$GD))
     MinGDGeno <- append(MinGDGeno,min(genoOut$GD))
@@ -227,24 +209,14 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
   }
   LogOut2 <- data.frame(Geno,MeanGDGeno,MinGDGeno,MaxGDGeno,MeanSNPsGeno)
   colnames(LogOut2) <- c("Genotype","Mean GDij|POk","Min GDij|POk","Max GDij|POk","Mean usable loci")
-  assign("Triad_Log2", LogOut2, envir = envir)
-  if (files==TRUE) {
-    blank <- data.frame(" "," "," "," "," "," ")
-    if(!missing(dir)) {
-      write.table(LogOut1,file.path(dir,"apparent-Triad-Summary.csv"),sep=",",col.names=T,row.names=F)
-      write.table(blank,file.path(dir,"apparent-Triad-Summary.csv"),sep=",",col.names=F,row.names=F,append=T)
-      write.table(blank,file.path(dir,"apparent-Triad-Summary.csv"),sep=",",col.names=F,row.names=F,append=T)
-      write.table(LogOut2,file.path(dir,"apparent-Triad-Summary.csv"),sep=",",col.names=T,row.names=F,append=T)
-    }
-  } 
   
   # Finding the GAP, testing its significance and calculating p-values.
   Out1a <- na.omit(Out1[order(Out1$GD),])
-  Out1b <- subset (Out1a, Out1a$GD <= MaxIdent)
-  Out1b <- subset (Out1b, Out1b$SNPs > nloci)
+  Out1b <- Out1a[which(Out1a$GD <= MaxIdent),]
+  Out1b <- Out1b[which(Out1b$SNPs > nloci),]
   
   if (self == FALSE) {
-    Out1b <- subset (Out1b, Out1b$Cross.Type != 'self')
+    Out1b <- Out1b[which(Out1b$Cross.Type != 'self'),]
   }
   
   if (nrow(Out1b) > 0) {
@@ -253,7 +225,7 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
       Tdiff[i] <- Out1b$GD[i+1] - Out1b$GD[i]
     }
   } else {
-    print ("No triads (pair of parents + offspring) were found.")
+    stop("No triads (pair of parents + offspring) were found.")
   }
   Tpv <- vector(mode="numeric",length=0)
   TMax <- max(na.omit(Tdiff))
@@ -263,8 +235,8 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
   
   if (TDtGap$p.value < alpha ) {
     TCutoff <- Out1b$GD[TIndex]
-    L <- subset (Out1b, Out1b$GD <= TCutoff)
-    H <- subset (Out1b, Out1b$GD > TCutoff)
+    L <- Out1b[which(Out1b$GD <= TCutoff),]
+    H <- Out1b[which(Out1b$GD > TCutoff),]
     
     if (nrow(L) > 0) {    
       if (nrow(H) > 29) {
@@ -283,13 +255,13 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
       # Significant triads
       Out2 <- data.frame(L, Tpv)
       colnames(Out2) <- c("Parent1","Parent2","Offspring","Cross.Type","SNPs","GD","p.value")
-      Out2a <- subset(Out2,Out2$p.value < alpha)
-
+      Out2a <- Out2[which(Out2$p.value < alpha),]
+      
       # Check hits with duplicated offspring and exclude them
       dupl.offs <- data.frame(unique(Out2a$Offspring[duplicated(Out2a$Offspring)]))
       if ((nrow(dupl.offs)) > 1) {
         for (i in 1:nrow(dupl.offs)){
-          Out1c <- subset (Out1b, Out1b$Offspring != dupl.offs[i,1])
+          Out1c <- Out1b[which(Out1b$Offspring != dupl.offs[i,1]),]
         }
         Tdiff <- vector(mode="numeric",length=0)  
         for (i in 1:nrow(Out1c)) {
@@ -303,8 +275,8 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
         
         if (TDtGap$p.value < alpha ) {
           TCutoff <- Out1c$GD[TIndex]
-          L <- subset (Out1c, Out1c$GD <= TCutoff)
-          H <- subset (Out1c, Out1c$GD > TCutoff)
+          L <- Out1c[which(Out1c$GD <= TCutoff),]
+          H <- Out1c[which(Out1c$GD > TCutoff),]
           
           if (nrow(L) > 0) {    
             if (nrow(H) > 29) {
@@ -325,255 +297,231 @@ apparent = function (InputFile, MaxIdent=0.10, alpha=0.01, nloci=300, self=TRUE,
       # Output 2 - Unique and significant Triads
       Out2 <- data.frame(L, Tpv)
       colnames(Out2) <- c("Parent1","Parent2","Offspring","Cross.Type","SNPs","GD","p.value")
-      Out2a <- subset(Out2,Out2$p.value < alpha)
+      Out2a <- Out2[which(Out2$p.value < alpha),]
       Out2a <- Out2a[order(Out2a$p.value),]
-      Triad_Sig <- Out2a
-      assign("Triad_Sig", Out2a, envir = envir)
-      if (files==TRUE){
-        if(!missing(dir)){
-          write.csv(Triad_Sig,file.path(dir, "apparent-Triad-Sig.csv"),row.names=F)
-        }
-      }
     }
   } else {
-    print("The Triad analysis GAP was not significant at the declared alpha level. No triads (pair of parents + offspring) were found.")
+    stop("The Triad analysis GAP was not significant at the declared alpha level. No triads (pair of parents + offspring) were found.")
   }
   
   # Print the Triad analysis plots
-  SortGD <- as.data.frame(na.omit(sort(GD)))
-  colnames(SortGD) <- "GD"
-  ThresholdT <- mean(c(SortGD[TIndex + 1,1], SortGD[TIndex,1]))
-  SortGD$Colour[SortGD$GD <= ThresholdT] = "red"
-  SortGD$Colour[SortGD$GD > ThresholdT] = "black"
-  
-  if (plot==TRUE & files==FALSE) {
+  if (plot==TRUE) {
+    SortGD <- as.data.frame(na.omit(sort(GD)))
+    colnames(SortGD) <- "GD"
+    ThresholdT <- mean(c(SortGD[TIndex + 1,1], SortGD[TIndex,1]))
+    SortGD$Colour[SortGD$GD <= ThresholdT] = "red"
+    SortGD$Colour[SortGD$GD > ThresholdT] = "black"
     plot (SortGD$GD,xlab="Test triads, ordered by GDij|POk",ylab="Gower Genetic Dissimilarity (GDij|k)",
           main="Triad analysis plot",pch=1,cex=.5,col=SortGD$Colour,xaxt="n")
     axis(1,at=c(1,nrow(SortGD)),labels=c("1",nrow(SortGD)),cex.axis=.7)
     abline(h = ThresholdT, lty = 2, col =  "tomato")
   
-  } else if (plot==TRUE & files==TRUE) {
-    png(file.path(dir,"apparent-Triad-Plot.png"), width=6,height=6,units='in',res=400)
-    par(mar = c(5,5,4,2), xpd = F,cex.axis=.6,cex.lab=.6,cex.main=.8)
-    plot (SortGD$GD,xlab="Test triads, ordered by GDij|POk",ylab="Gower Genetic Dissimilarity (GDij|k)",
-          main="Triad analysis plot",pch=1,cex=.5,col=SortGD$Colour,xaxt="n")
-    axis(1,at=c(1,nrow(SortGD)),labels=c("1",nrow(SortGD)),cex.axis=.7)
-    abline(h = ThresholdT, lty = 2, col =  "tomato")
-    dev.off()
   }
 
   ##########################################################
   # Dyad analysis - for the Triad analysis non-assignments
   ##########################################################
-  
   if (Dyad==TRUE) {
-    print ("Starting the Dyad analysis...")
     AllOffs <- as.character(colnames(Offsprings))
     OffsAssigned <- as.character(unique(Out2a[,3]))
     OffsD <- as.data.frame(setdiff(AllOffs,OffsAssigned))
     
     if (nrow(OffsD) < 1) {
       stop ("No Dyad analysis is needed because the Triad analysis identified the parents for all tested offspring. Please refer to the output file apparent-Triad-Sig.txt.")
-    } else {
-      ParD <- as.data.frame(unique(Out1[,1]))
-      colnames(OffsD) <- "X"
-      colnames(ParD) <- "X"
-      GDM <- data.frame(matrix(ncol=nrow(OffsD),nrow=nrow(ParD)))
-      GDCV <- data.frame(matrix(ncol=nrow(OffsD),nrow=nrow(ParD)))
+    }
+    
+    ParD <- as.data.frame(unique(Out1[,1]))
+    colnames(OffsD) <- "X"
+    colnames(ParD) <- "X"
+    GDM <- data.frame(matrix(ncol=nrow(OffsD),nrow=nrow(ParD)))
+    GDCV <- data.frame(matrix(ncol=nrow(OffsD),nrow=nrow(ParD)))
       
-      for (i in 1:nrow(ParD)) {  
-        Pa1 <- Out1[grep(ParD[i,1],Out1$Parent1),c(1,3,6)]
-        Pa2 <- Out1[grep(ParD[i,1],Out1$Parent2),c(2,3,6)]
-        colnames (Pa1) <- c("P","O","GD")
-        colnames (Pa2) <- c("P","O","GD")
-        Pa1Pa2 <- rbind(Pa1,Pa2)  
-        for (j in 1:nrow(OffsD)) {
-          Samp <- head(Pa1Pa2[grep(OffsD$X[j],Pa1Pa2$O),3],-1)
-          avg <- mean(Samp)
-          dev <- sd(Samp)
+    for (i in 1:nrow(ParD)) {  
+      Pa1 <- Out1[grep(ParD[i,1],Out1$Parent1),c(1,3,6)]
+      Pa2 <- Out1[grep(ParD[i,1],Out1$Parent2),c(2,3,6)]
+      colnames (Pa1) <- c("P","O","GD")
+      colnames (Pa2) <- c("P","O","GD")
+      Pa1Pa2 <- rbind(Pa1,Pa2)  
+      for (j in 1:nrow(OffsD)) {
+        Samp <- head(Pa1Pa2[grep(OffsD$X[j],Pa1Pa2$O),3],-1)
+        avg <- mean(Samp)
+        dev <- sd(Samp)
           
-          # Calculating GDM and GDCV values
-          dat <- ConvertedData[c(as.character(ParD[i,]),as.character(OffsD[j,]))]
-          dat <- na.omit(dat)
-          colnames(dat) <- c("geno1","geno2")
-          dat$diff <- abs(dat$geno1 - dat$geno2)
-          dist <- sum(dat$diff) / length(dat$diff)
-          ratio <- dev/dist      
-          GDM[i,j] <- avg
-          GDCV[i,j] <- ratio
-          colnames(GDM) <- as.character(OffsD$X)
-          rownames(GDM) <- as.character(ParD$X)
-          colnames(GDCV) <- as.character(OffsD$X)
-          rownames(GDCV) <- as.character(ParD$X)
-        } 
+        # Calculating GDM and GDCV values
+        dat <- ConvertedData[c(as.character(ParD[i,]),as.character(OffsD[j,]))]
+        dat <- na.omit(dat)
+        colnames(dat) <- c("geno1","geno2")
+        dat$diff <- abs(dat$geno1 - dat$geno2)
+        dist <- sum(dat$diff) / length(dat$diff)
+        ratio <- dev/dist      
+        GDM[i,j] <- avg
+        GDCV[i,j] <- ratio
+        colnames(GDM) <- as.character(OffsD$X)
+        rownames(GDM) <- as.character(ParD$X)
+        colnames(GDCV) <- as.character(OffsD$X)
+        rownames(GDCV) <- as.character(ParD$X)
+      } 
+    }
+      
+    # Assign p-values to each significant parent-offspring pair   
+    DPa <- vector(mode="numeric",length=0)
+    DOf <- vector(mode="numeric",length=0)
+    DMPv <- vector(mode="numeric",length=0)
+    DRPv <- vector(mode="numeric",length=0)
+    DCumPv <- vector(mode="numeric",length=0)
+      
+    for (i in 1:ncol(GDM)) {
+      M <- cbind(as.matrix(rownames(GDM)),as.data.frame(GDM[,i]))
+      colnames(M) <- c("P","Mean")
+      M <- na.omit(M[order(M$Mean),])      
+      R <- cbind(as.matrix(rownames(GDCV)),as.data.frame(GDCV[,i]))
+      colnames(R) <- c("P","Ratio")
+      R <- na.omit(R[order(R$Ratio),])
+      
+      # Testing GDMs
+      Mp <- as.data.frame(pnorm ((M$Mean - mean(M$Mean)) / sd (M$Mean)))
+      MpVect <- cbind(M$P,Mp)
+      colnames(MpVect) <- c("P","Mpnorm")
+      MpSig <- MpVect[which(MpVect$Mpnorm < sqrt(alpha)),]
+      if (nrow(MpSig) < 1) {
+        next
       }
-      
-      # Assign p-values to each significant parent-offspring pair   
-      DPa <- vector(mode="numeric",length=0)
-      DOf <- vector(mode="numeric",length=0)
-      DMPv <- vector(mode="numeric",length=0)
-      DRPv <- vector(mode="numeric",length=0)
-      DCumPv <- vector(mode="numeric",length=0)
-      
-      for (i in 1:ncol(GDM)) {
-        M <- cbind(as.matrix(rownames(GDM)),as.data.frame(GDM[,i]))
-        colnames(M) <- c("P","Mean")
-        M <- na.omit(M[order(M$Mean),])      
-        R <- cbind(as.matrix(rownames(GDCV)),as.data.frame(GDCV[,i]))
-        colnames(R) <- c("P","Ratio")
-        R <- na.omit(R[order(R$Ratio),])
         
-        # Testing GDMs
-        Mp <- as.data.frame(pnorm ((M$Mean - mean(M$Mean)) / sd (M$Mean)))
-        MpVect <- cbind(M$P,Mp)
-        colnames(MpVect) <- c("P","Mpnorm")
-        MpSig <- subset(MpVect,MpVect$Mpnorm < sqrt(alpha))
-        if (nrow(MpSig) < 1) {
-          next
-        }
+      # Testing GDCVs
+      Rp <- as.data.frame(pnorm ((R$Ratio - mean(R$Ratio)) / sd (R$Ratio)))
+      RpVect <- cbind(R$P,Rp)
+      colnames(RpVect) <- c("P","Rpnorm")
+      RpSig <- RpVect[which(RpVect$Rpnorm > (1 - sqrt(alpha))),]
         
-        # Testing GDCVs
-        Rp <- as.data.frame(pnorm ((R$Ratio - mean(R$Ratio)) / sd (R$Ratio)))
-        RpVect <- cbind(R$P,Rp)
-        colnames(RpVect) <- c("P","Rpnorm")
-        RpSig <- subset(RpVect,RpVect$Rpnorm > (1 - sqrt(alpha)))
+      # Genotype(s) that passed on both GDM and GDCV tests
+      MRSig <- merge(MpSig,RpSig, by = "P")
         
-        # Genotype(s) that passed on both GDM and GDCV tests
-        MRSig <- merge(MpSig,RpSig, by = "P")
-        
-        if (nrow(MRSig) < 1) {
-          next
-        } else {
-          for (j in 1: nrow(MRSig)) {
-            DPa <- append(DPa,as.character(MRSig$P[j]))
-            DOf <- append(DOf,colnames(GDM[i]))
-            DMPv <- append(DMPv,MRSig$Mpnorm[j])
-            DRPv <- append(DRPv,(1 - MRSig$Rpnorm[j]))
-            DCumPv <- append(DCumPv,(MRSig$Mpnorm[j] * (1 - MRSig$Rpnorm[j])))
-          }     
-        }
+      if (nrow(MRSig) < 1) {
+        next
+      } else {
+        for (j in 1: nrow(MRSig)) {
+          DPa <- append(DPa,as.character(MRSig$P[j]))
+          DOf <- append(DOf,colnames(GDM[i]))
+          DMPv <- append(DMPv,MRSig$Mpnorm[j])
+          DRPv <- append(DRPv,(1 - MRSig$Rpnorm[j]))
+          DCumPv <- append(DCumPv,(MRSig$Mpnorm[j] * (1 - MRSig$Rpnorm[j])))
+        }     
       }
+    }
       
-      # Dyad analysis output 
-      Out3 <- data.frame(DPa,DOf,DMPv,DRPv,DCumPv)
-      Out3 <- subset(Out3, ((Out3$DMPv < alpha) & (Out3$DCumPv < alpha)) | ((Out3$DRPv < alpha) & (Out3$DCumPv < alpha)) )
-      Out3 <- Out3[order(Out3$DCumPv),]
+    # Dyad analysis output 
+    Out3 <- data.frame(DPa,DOf,DMPv,DRPv,DCumPv)
+    Out3 <- Out3[which( ((Out3$DMPv < alpha) & (Out3$DCumPv < alpha)) | ((Out3$DRPv < alpha) & (Out3$DCumPv < alpha)) ),]
+    Out3 <- Out3[order(Out3$DCumPv),]
       
-      if (nrow(Out3) < 1) {
-        stop ("At the declared alpha level, the Dyad analysis was unable to find any signficant parent-offspring associations.")
+    if (nrow(Out3) < 1) {
+      stop ("At the declared alpha level, the Dyad analysis was unable to find any signficant parent-offspring associations.")
       
-      } else if (nrow(Out3) == 1) {
-        colnames(Out3) <- c("Parent","Offspring","GDM p-value","GDCV p-value","Cumulative p-value")
-        Dyad_Sig <- Out3
-        assign("Dyad_Sig", Out3, envir = envir)
-        Out3b <- Out3
-        if (files==TRUE){
-          if(!missing(dir)){
-            write.csv(Out3,file.path(dir, "apparent-Dyad-Sig.csv"),row.names=F)
-          }
-        }
-        
-      } else if (nrow(Out3) > 1) {
-        # Skip all parent-offspring pairs already considered, as well as all offspring successfuly assigned to parental pairs in the Triad analysis.     
-        Out3a <- data.frame(matrix(ncol=7,nrow=0))
-        Out3b <- data.frame(matrix(ncol=5,nrow=0))
-        Out3$D1 <- paste(Out3$DPa,Out3$DOf,sep=".")
-        Out3$D2 <- paste(Out3$DOf,Out3$DPa,sep=".")
-        
-        # If a duplicated analysis, keep the likely one (strongest evidence > 200)
-        for (i in 1:nrow(Out3)) {
-          if (Out3$D2[i] %in% Out3$D1) {
-            j <- grep(Out3$D2[i], Out3$D1)
-            if ( (Out3[j,5] / Out3[i,5] ) > 200 ) {
-              Out3a <- rbind (Out3a,Out3[i,])
-            } else {
-              Out3a <- rbind (Out3a,Out3[j,])
-            }
+    } else if (nrow(Out3) == 1) {
+      colnames(Out3) <- c("Parent","Offspring","GDM p-value","GDCV p-value","Cumulative p-value")
+      Out3b <- Out3
+
+    } else if (nrow(Out3) > 1) {
+      # Skip all parent-offspring pairs already considered, as well as all offspring successfuly assigned to parental pairs in the Triad analysis.     
+      Out3a <- data.frame(matrix(ncol=7,nrow=0))
+      Out3b <- data.frame(matrix(ncol=5,nrow=0))
+      Out3$D1 <- paste(Out3$DPa,Out3$DOf,sep=".")
+      Out3$D2 <- paste(Out3$DOf,Out3$DPa,sep=".")
+      
+      # If a duplicated analysis, keep the likely one (strongest evidence > 200)
+      for (i in 1:nrow(Out3)) {
+        if (Out3$D2[i] %in% Out3$D1) {
+          j <- grep(Out3$D2[i], Out3$D1)
+          if ( (Out3[j,5] / Out3[i,5] ) > 200 ) {
             Out3a <- rbind (Out3a,Out3[i,])
-          }
-        }
-        Out3a <- Out3a[order(Out3a$DOf,Out3a$DCumPv),]
-        
-        for (i in 1:nrow(Out3a)) {
-          if (i == nrow(Out3a)) {
-            break
           } else {
-            Occur <- length(grep(Out3a$DOf[i],Out3a$DOf))     
-            if (Occur == 1) {
+            Out3a <- rbind (Out3a,Out3[j,])
+          }
+          Out3a <- rbind (Out3a,Out3[i,])
+        }
+      }
+      Out3a <- Out3a[order(Out3a$DOf,Out3a$DCumPv),]
+      
+      for (i in 1:nrow(Out3a)) {
+        if (i == nrow(Out3a)) {
+          break
+        } else {
+          Occur <- length(grep(Out3a$DOf[i],Out3a$DOf))     
+          if (Occur == 1) {
+            Out3b <- rbind (Out3b,Out3a[i,c(1:5)])
+          } else if (Occur > 1) {
+            if ( (Out3a[i+1,2] != Out3a[i,2]) && ((Out3a[i+1,5] / Out3a[i,5]) >= 200) ) {
               Out3b <- rbind (Out3b,Out3a[i,c(1:5)])
-            } else if (Occur > 1) {
-              if ( (Out3a[i+1,2] != Out3a[i,2]) && ((Out3a[i+1,5] / Out3a[i,5]) >= 200) ) {
-                Out3b <- rbind (Out3b,Out3a[i,c(1:5)])
-              }
             }
           }
         }
-        Out3b <- Out3b[,1:5]
-        Out3b <- Out3b[order(Out3b$DCumPv),]
-        colnames(Out3b) <- c("Parent","Offspring","GDM p-value","GDCV p-value","Cumulative p-value")
-        Dyad_Sig <- Out3b
-        assign("Dyad_Sig", Out3b, envir = envir)
-        if (files==TRUE){
-          if(!missing(dir)){
-            write.csv(Dyad_Sig,file.path(dir, "apparent-Dyad-Sig.csv"),row.names=F)
-          }
-        }
       }
+      Out3b <- Out3b[,1:5]
+      Out3b <- Out3b[order(Out3b$DCumPv),]
+      colnames(Out3b) <- c("Parent","Offspring","GDM p-value","GDCV p-value","Cumulative p-value")
+    }
       
-      # Plotting the Dyad results
-      if ( plot==TRUE & files==TRUE & nrow(Out3b > 0) ) {
-        pdf(file.path(dir,"apparent-Dyad-Plot.pdf"),width=7,height=4)
-        for (i in 1:nrow(Out3b)) {
-          par(mfrow=c(2,1))
-          par(mar=c(2,1,1,1))
-          par(mgp=c(.5,.5,0))
-          par(oma=c(0,0,0,0))
-          PlotM <- as.data.frame(GDM[,as.character(Out3b$Offspring[i])])
-          PlotR <- as.data.frame(GDCV[,as.character(Out3b$Offspring[i])])
-          PlotM <- cbind(rownames(GDM),PlotM)
-          PlotR <- cbind(rownames(GDCV),PlotR)
-          colnames(PlotM) <- c("P","Mean")
-          colnames(PlotR) <- c("P","Ratio")
-          PlotM <- na.omit(PlotM[order(PlotM$Mean),])
-          PlotR <- na.omit(PlotR[order(PlotR$Ratio),])
-          # Normal probabilities of Means (GDM)
-          PlotMpnorm <- as.data.frame(pnorm((PlotM$Mean - mean(PlotM$Mean)) / sd (PlotM$Mean)))
-          PlotM <- cbind(PlotM,PlotMpnorm)
-          colnames(PlotM) <- c("P","Mean","Mpnorm")
-          PlotM$Colour[PlotM$Mpnorm <= sqrt(alpha)] = "red"
-          PlotM$Colour[PlotM$Mpnorm > sqrt(alpha)] = "black"
-          M_axis <- PlotM[c(1,nrow(PlotM)),c(1,2)]
-          cntM <- grep("red",PlotM$Colour)
-          M_lower_bound <- -qnorm(1 - sqrt(alpha)) * sd(PlotM$Mean) + mean(PlotM$Mean)
-          # # Normal probabilities of Ratio (GDCV)
-          PlotRpnorm <- as.data.frame(pnorm((PlotR$Ratio - mean(PlotR$Ratio)) / sd (PlotR$Ratio)))
-          PlotR <- cbind(PlotR,PlotRpnorm)
-          colnames(PlotR) <- c("P","Ratio","Rpnorm")
-          PlotR$Colour[PlotR$Rpnorm <= 1-(sqrt(alpha))] = "black"
-          PlotR$Colour[PlotR$Rpnorm > 1-(sqrt(alpha))] = "red"
-          R_axis = PlotR[c(1,nrow(PlotR)),c(1,2)]
-          cntR <- grep("red",PlotR$Colour)
-          R_upper_bound <- qnorm(1 - sqrt(alpha)) * sd(PlotR$Ratio) + mean(PlotR$Ratio)
-          # GDM plot
-          plot (PlotM$Mean,rep(1,nrow(PlotM)),xlab="GDM",col=PlotM$Colour,ylab="", 
-                main=Out3b$Offspring[i],pch=1,cex=.5,cex.lab=.7,axes=F,line=-1)
-          axis(side=1,at=M_axis$Mean,labels=round(M_axis$Mean,2),cex.axis=.5,line=-3)
-          segments(x0=M_lower_bound, y0=.85, x1=M_lower_bound, y1=1,col='red',lty=3)
-          #text(M_text,.7,labels="GDM",pos=3,cex=.4,offset=.1)
-          text(PlotM$Mean[cntM],1,labels=PlotM$P[cntM],pos=4,cex=.4,srt=45,offset=.3)
-          text(M_lower_bound,.7,labels="Lower bound\ncutoff",pos=3,cex=.4,offset=.3)
-          # GDCV plot
-          plot (PlotR$Ratio,rep(1,nrow(PlotR)),xlab="GDCV",col=PlotR$Colour,ylab="",
-                pch=1,cex=.5,cex.lab=.7,axes=F,line=-1)
-          axis(side=1,at=R_axis$Ratio,labels=round(R_axis$Ratio,2),cex.axis=.5,line=-3)
-          segments(x0=R_upper_bound, y0=.85, x1=R_upper_bound, y1=1,col='red',lty=3)
-          text(PlotR$Ratio[cntR],1,labels=PlotR$P[cntR],pos=2,cex=.4,srt=315,offset=.3)
-          text(R_upper_bound,.7,labels="Upper bound\ncutoff",pos=3,cex=.4,offset=.3)
-        }
-        dev.off()
+    # Plotting the Dyad results
+    if ( plot==TRUE & nrow(Out3) > 0 ) {
+      for (i in 1:nrow(Out3b)) {
+        par(mfrow=c(2,1))
+        par(mar=c(2,1,1,1))
+        par(mgp=c(.5,.5,0))
+        par(oma=c(0,0,0,0))
+        PlotM <- as.data.frame(GDM[,as.character(Out3b$Offspring[i])])
+        PlotR <- as.data.frame(GDCV[,as.character(Out3b$Offspring[i])])
+        PlotM <- cbind(rownames(GDM),PlotM)
+        PlotR <- cbind(rownames(GDCV),PlotR)
+        colnames(PlotM) <- c("P","Mean")
+        colnames(PlotR) <- c("P","Ratio")
+        PlotM <- na.omit(PlotM[order(PlotM$Mean),])
+        PlotR <- na.omit(PlotR[order(PlotR$Ratio),])
+        # Normal probabilities of Means (GDM)
+        PlotMpnorm <- as.data.frame(pnorm((PlotM$Mean - mean(PlotM$Mean)) / sd (PlotM$Mean)))
+        PlotM <- cbind(PlotM,PlotMpnorm)
+        colnames(PlotM) <- c("P","Mean","Mpnorm")
+        PlotM$Colour[PlotM$Mpnorm <= sqrt(alpha)] = "red"
+        PlotM$Colour[PlotM$Mpnorm > sqrt(alpha)] = "black"
+        M_axis <- PlotM[c(1,nrow(PlotM)),c(1,2)]
+        cntM <- grep("red",PlotM$Colour)
+        M_lower_bound <- -qnorm(1 - sqrt(alpha)) * sd(PlotM$Mean) + mean(PlotM$Mean)
+        # # Normal probabilities of Ratio (GDCV)
+        PlotRpnorm <- as.data.frame(pnorm((PlotR$Ratio - mean(PlotR$Ratio)) / sd (PlotR$Ratio)))
+        PlotR <- cbind(PlotR,PlotRpnorm)
+        colnames(PlotR) <- c("P","Ratio","Rpnorm")
+        PlotR$Colour[PlotR$Rpnorm <= 1-(sqrt(alpha))] = "black"
+        PlotR$Colour[PlotR$Rpnorm > 1-(sqrt(alpha))] = "red"
+        R_axis = PlotR[c(1,nrow(PlotR)),c(1,2)]
+        cntR <- grep("red",PlotR$Colour)
+        R_upper_bound <- qnorm(1 - sqrt(alpha)) * sd(PlotR$Ratio) + mean(PlotR$Ratio)
+        # GDM plot
+        plot (PlotM$Mean,rep(1,nrow(PlotM)),xlab="GDM",col=PlotM$Colour,ylab="", 
+              main=Out3b$Offspring[i],pch=1,cex=.5,cex.lab=.7,axes=F)
+        axis(side=1,at=M_axis$Mean,labels=round(M_axis$Mean,2),cex.axis=.5,line=-3)
+        segments(x0=M_lower_bound, y0=.85, x1=M_lower_bound, y1=1,col='red',lty=3)
+        #text(M_text,.7,labels="GDM",pos=3,cex=.4,offset=.1)
+        text(PlotM$Mean[cntM],1,labels=PlotM$P[cntM],pos=4,cex=.4,srt=45,offset=.3)
+        text(M_lower_bound,.7,labels="Lower bound\ncutoff",pos=3,cex=.4,offset=.3)
+        # GDCV plot
+        plot (PlotR$Ratio,rep(1,nrow(PlotR)),xlab="GDCV",col=PlotR$Colour,ylab="",
+              pch=1,cex=.5,cex.lab=.7,axes=F)
+        axis(side=1,at=R_axis$Ratio,labels=round(R_axis$Ratio,2),cex.axis=.5,line=-3)
+        segments(x0=R_upper_bound, y0=.85, x1=R_upper_bound, y1=1,col='red',lty=3)
+        text(PlotR$Ratio[cntR],1,labels=PlotR$P[cntR],pos=2,cex=.4,srt=315,offset=.3)
+        text(R_upper_bound,.7,labels="Upper bound\ncutoff",pos=3,cex=.4,offset=.3)
       }
     }
   }
-  print("Done.")
-  proc.time() - ptm
+  
+  ########################################################################
+  # Creating the list of outputs and return it, with or w/o Dyad analysis
+  ########################################################################
+  if (Dyad==TRUE){
+    combined_output <- list(Triad_all = Out1, Triad_sig = Out2a, Triad_summary_pop = LogOut1,
+                            Triad_summary_geno = LogOut2, Dyad_sig = Out3b)
+  } else {
+    combined_output <- list(Triad_all = Out1, Triad_sig = Out2a, Triad_summary_pop = LogOut1,
+                          Triad_summary_geno = LogOut2)
+  }
+  return (combined_output)
 } 
